searchData={"content_type":"text/plain","items":[{"doc":null,"ref":"fsm_example.html","title":"fsm_example","type":"module"},{"doc":null,"ref":"fsm_example.html#a/1","title":"fsm_example.a/1","type":"function"},{"doc":null,"ref":"fsm_example.html#b/1","title":"fsm_example.b/1","type":"function"},{"doc":null,"ref":"fsm_example.html#code_change/3","title":"fsm_example.code_change/3","type":"function"},{"doc":null,"ref":"fsm_example.html#data_vsn/0","title":"fsm_example.data_vsn/0","type":"function"},{"doc":null,"ref":"fsm_example.html#spawn_link/0","title":"fsm_example.spawn_link/0","type":"function"},{"doc":"A behaviour/support library for writing plain Erlang FSMs. This module implements an OTP behaviour for writing plain Erlang FSMs, alleviating a long-standing gripe of mine that the OTP behaviours, for all their power, force programmers into a coding style that is very much different from that taught in the Basic Erlang Course (or the book, or online tutorials, ...) -- the type of programming that made us want to use Erlang in the first place. Only in my old age have I begun to understand fully what a sacrifice this is. See e.g. my presentation Death by Accidental Complexity (QCon SF 2010) for a more detailed discussion of the issues involved. (Slides also available in the doc/ directory of this repos) The requirements that drove us away from plain Erlang programming in the first place were: The need to support system messages to control upgrade, state inspection, shutdown, etc. The plain_fsm library solves this in a reasonable way, I think. The need for debugging support . The debugging support in e.g. gen_server is, I believe, rendered obsolete by the new powerful trace support (and dbg) in later versions of OTP. In the case of gen_server, reducing the need to reinvent the wheel , a valid point, but more so for e.g. the client side of gen_server:call(). In a protocol state machine, the only thing that really needs reusing is the handling of system messages. However, the behaviours provided by OTP for FSM programming, gen_server and gen_fsm ( gen_server is perhaps a more common choice than gen_fsm ), both have the distinct drawback that you cannot normally start with a classic Erlang design and then migrate to a behaviour without significant code rewrite. In addition, the two behaviours are semantically different from the classic Erlang design Using plain_fsm First, write your state machine without worrying about OTP system messages. Once you're happy with it, figure out where you really want to handle system messages. Normally, it will suffice to do it in a fairly stable state. A good rule of thumb is that the top-level state machine should handle system messages, while the transient (sub-) states shouldn't In the states where you want to handle system messages, you have three choices: (A) Insert the system messages in the receive clause: idle ( S ) -&gt; Parent = plain_fsm : info ( parent ) , receive { system , From , Req } -&gt; plain_fsm : handle_system_msg ( Req , From , S , fun ( S1 ) -&gt; idle ( S1 ) end ) ; { &#39;EXIT&#39; , Parent , Reason } -&gt; plain_fsm : parent_EXIT ( Reason , S ) ; . . . %% your original code here end . This has the advantage that everyone can understand what's going on. The part that plain_fsm.erl helps you with is the set of functions system_code_change() , system_continue() , system_shutdown() , format_status() , which are required callbacks when you handle system messages directly. (B) Handle system messages and unknown messages together: idle ( S ) -&gt; Parent = plain_fsm : info ( parent ) , receive . . . %% your original code here Msg -&gt; plain_fsm : handle_msg ( Msg , State , fun ( S1 ) -&gt; idle ( S1 ) end ) end . This is quite convenient if the receive statement already has a 'catch-all' clause, discarding unknown messages. plain_fsm:handle_msg/3 will handle system messages properly and ignore any other message. (C) Write a pseudo wrapper function around your receive clause: idle ( S ) -&gt; plain_fsm : extended_receive ( receive . . . %% your original code end ) . The function plain_fsm:extended_receive/1 is replaced in a parse_transform into something that looks very much like the previous program (A). The code, as it reads, requires the reader to know that the transformation takes place, otherwise the semantics would be confusing (you cannot solve the problem using a real function that way.) On the plus side, this is a fairly small violation of both the original code and Erlang's semantics. Note that for this to work, you must include &quot;plain_fsm.hrl&quot; in your module. Example In the module fsm_example.erl (included in the plain_fsm package), we choose to handle system messages in the idle state. The example code is runnable, and supports suspend, resume, status inspection, and code change. Imagine that the code initially looked like this: idle ( S ) -&gt; receive a -&gt; io : format ( &quot;going to state a ~n &quot; , [ ] ) , a ( S ) ; b -&gt; io : format ( &quot;going to state b ~n &quot; , [ ] ) , b ( S ) after 10000 -&gt; io : format ( &quot;timeout in idle ~n &quot; , [ ] ) , idle ( S ) end ) . The change required to handle system messages is as follows: idle(S) -&gt; plain_fsm:extended_receive ( receive a -&gt; io:format(&quot;going to state a~n&quot;, []), a(S); b -&gt; io:format(&quot;going to state b~n&quot;, []), b(S) after 10000 -&gt; io:format(&quot;timeout in idle~n&quot;, []), idle(S) end). In addition, we change the start function from, in this case: spawn_link ( ) -&gt; spawn_link ( fun ( ) -&gt; process_flag ( trap_exit , true ) , idle ( mystate ) end ) . Is changed into: spawn_link() -&gt; plain_fsm:spawn_link (?MODULE, fun() -&gt; process_flag(trap_exit,true), idle(mystate) end). See also spawn/2 and spawn_opt/3 for information on other possible start functions. To be fully compliant, you also need to supply a code_change/3 function. See behaviour_info/1 for details.","ref":"plain_fsm.html","title":"plain_fsm","type":"behaviour"},{"doc":null,"ref":"plain_fsm.html#c:code_change/3","title":"plain_fsm.code_change/3","type":"callback"},{"doc":"Virtual function for extracting the current function. This function call is expanded by the plain_fsm parse transform into the name and arity ( {Module, Function, Arity} ) of the current function. It cannot be used from code that hasn't been transformed.","ref":"plain_fsm.html#current_function/0","title":"plain_fsm.current_function/0","type":"function"},{"doc":null,"ref":"plain_fsm.html#c:data_vsn/0","title":"plain_fsm.data_vsn/0","type":"callback"},{"doc":"Virtual function used to wrap receive clauses. This function cannot be called directly, but is intended as a syntactic wrapper around a receive clause. It will be transformed at compile time to a set of receive patterns handling system messages and parent termination according to the OTP rules. The transform requires that the surrounding function has exactly one argument (the &quot;State&quot; or &quot;Loop Data&quot;.) To trigger the parse_transform, include the file plain_fsm.hrl (found in plain_fsm/inc/ ) in your module, and the Erlang compiler must be able to find the module plain_fsm_xform.beam . If erlc is used, this is accomplished by adding -pa .../plain_fsm/ebin to the erlc command.","ref":"plain_fsm.html#extended_receive/1","title":"plain_fsm.extended_receive/1","type":"function"},{"doc":null,"ref":"plain_fsm.html#format_status/2","title":"plain_fsm.format_status/2","type":"function"},{"doc":null,"ref":"plain_fsm.html#c:format_status/2","title":"plain_fsm.format_status/2","type":"callback"},{"doc":"Called in a &quot;catch-all&quot; clause within a receive statement. This function never returns. It will handle system messages properly and ignore anything else. Example: idle ( S ) -&gt; receive . . . Msg -&gt; plain_fsm : handle_msg ( Msg , S , fun ( S1 ) -&gt; idle ( S1 ) end ) end . Note that this function should only be used if it is known to be safe to discard unknown messages. In most state machines there should be at least one state where unknown messages are discarded; in these states, the handle_msg/3 function can be a convenient way to handle both unknown messages and system messages. The Cont argument should be either a fun with one argument (the new state), which jumps back into the user code in the proper place, or it can be the name of a function (in this case, 'idle'). In the latter case, the function in question must be exported; in the former case, this is not necessary.","ref":"plain_fsm.html#handle_msg/3","title":"plain_fsm.handle_msg/3","type":"function"},{"doc":"Called when the process receives a system message. This function never returns. If the program handles system messages explicitly, this function can be called to handle them in the plain_fsm way. Example: idle ( S ) -&gt; receive { system , From , Req } -&gt; plain_fsm : handle_system_msg ( Req , From , S , fun ( S1 ) -&gt; idle ( S1 ) end ) ; . . . end . The Cont argument should be either a fun with one argument (the new state), which jumps back into the user code in the proper place, or it can be the name of a function (in this case, 'idle'). In the latter case, the function in question must be exported; in the former case, this is not necessary.","ref":"plain_fsm.html#handle_system_msg/4","title":"plain_fsm.handle_system_msg/4","type":"function"},{"doc":"Virtual function used to wrap a call to the BIF erlang:hibernate/3. This function cannot be called directly, but translates to the call erlang:hibernate(plain_fsm,wake_up,[data_vsn(),Module,M,F,A]) where Module:data_vsn() and Module:code_change/3 are expected to exist (the parse_transform will add and export the function data_vsn() -&lt; 0 , if it doesn't already exist.) The function plain_fsm:wake_up/5 will begin by calling Module:data_vsn() , and if it is the same as before, simply call apply(M,F,A) . Otherwise, Module:code_change(OldVsn, IntState, hibernate) will be called first. This allows a plain_fsm behaviour module to be &quot;bootstrapped&quot; to a new version during hibernation.","ref":"plain_fsm.html#hibernate/3","title":"plain_fsm.hibernate/3","type":"function"},{"doc":"retrieves meta-data for the plain_fsm process. Description of available meta-data: debug : See the manual for sys . erl name : Internal name , normally the same as the registered name . initially undefined , can be set via plain_fsm : store_name / 1 . mod : Name of the callback module . parent : The pid ( ) of the parent process .","ref":"plain_fsm.html#info/1","title":"plain_fsm.info/1","type":"function"},{"doc":"Handles parent termination properly. This function is called when the parent of a plain_fsm instance dies. The OTP rules state that the child should die with the same reason as the parent (especially in the case of Reason='shutdown'.) It is possible to provide a function terminate in the callback module. If such function is exported, it will be called as Mod:terminate(Reason, State) . This behaviour is borrowed from sys.erl.","ref":"plain_fsm.html#parent_EXIT/2","title":"plain_fsm.parent_EXIT/2","type":"function"},{"doc":"Equivalent to proc_lib:spawn(StartF) . This function also initializes the plain_fsm meta-data.","ref":"plain_fsm.html#spawn/2","title":"plain_fsm.spawn/2","type":"function"},{"doc":"Equivalent to proc_lib:spawn_link(StartF) . This function also initializes the plain_fsm meta-data.","ref":"plain_fsm.html#spawn_link/2","title":"plain_fsm.spawn_link/2","type":"function"},{"doc":"Equivalent to proc_lib:spawn_opt(StartF, Opts) . This function also initializes the plain_fsm meta-data.","ref":"plain_fsm.html#spawn_opt/3","title":"plain_fsm.spawn_opt/3","type":"function"},{"doc":"Equivalent to proc_lib:spawn_opt(Node, StartF, Opts) . This function also initializes the sysFsm meta-data.","ref":"plain_fsm.html#spawn_opt/4","title":"plain_fsm.spawn_opt/4","type":"function"},{"doc":"Similar to proc_lib:start(M,F,A, Timeout, Opts) . This function works in a similar fashion to proc_lib:start/5 , but takes a fun instead of a {M,F,A} argument. InitF() may return one of the following: * {reply, Reply, Cont} , where Reply will be sent back to the parent, and Cont is a continuation function with no arguments. * {noreply, Cont} , which sends no ack message back to the parent (presumably, this is done elsewhere in the code then).","ref":"plain_fsm.html#start_opt/4","title":"plain_fsm.start_opt/4","type":"function"},{"doc":"stores an internal name for the FSM (for sys:get_status() ). This can be used if the FSM were started as an anonymous process (the only kind currently supported). Note that this function does not register the name. The name stored is the one that shows up in sys:get_status/1. No restriction is made here regarding the data type.","ref":"plain_fsm.html#store_name/1","title":"plain_fsm.store_name/1","type":"function"},{"doc":null,"ref":"plain_fsm.html#system_code_change/4","title":"plain_fsm.system_code_change/4","type":"function"},{"doc":null,"ref":"plain_fsm.html#system_continue/3","title":"plain_fsm.system_continue/3","type":"function"},{"doc":"Internal export; called in order to retrieve the internal state. This function is called through sys:get_state/1 . See also system_replace_state/2 . Note that the internal state is represented as {Options, State} .","ref":"plain_fsm.html#system_get_state/1","title":"plain_fsm.system_get_state/1","type":"function"},{"doc":"Internal export; called in order to update internal state. This function is called through sys:replace_state/2 . Note that the external representation of the state is {Options, State} , and the options return is the 'processed' options list, possibly ignoring some elements provided by StateFun . See also system_get_state/1 .","ref":"plain_fsm.html#system_replace_state/2","title":"plain_fsm.system_replace_state/2","type":"function"},{"doc":null,"ref":"plain_fsm.html#system_terminate/4","title":"plain_fsm.system_terminate/4","type":"function"},{"doc":"Helper function to dispatch blocking calls as tail calls. During code change, it can be a problem that processes lie in blocking calls - say, e.g., to gen_tcp:connect(...) . If the module is reloaded, the calling function will still be on the call stack, and may eventually get the process killed (as the VM only holds two versions of the module). This function is most easily called using the macro ?tail_apply(F, ContF, S) , which expands to plain_fsm : tail_apply ( F , ? MODULE : data_vsn ( ) , ? MODULE , ContF , S ) In this case, ?MODULE:data_vsn() will have been automatically generated by plain_fsm, or is manually updated whenever the internal representation of the state S is changed. ContF represents an exported function in the calling module, ContF(Status, Result, S) Status :: ok | error Result :: fun() | any() If the call to Fun() fails, the exception (throw, error or exit) will be caught, and Result will be a fun (arity 0), which can be called to &quot;re-throw&quot; the exception. This way, the continuation function can catch exceptions in its own try/catch pattern. 'Status' will be error if Fun() fails, otherwise ok . Thus, the simplest implementation of ContF would be: ContF ( ok , Result , S ) -&gt; handle_result ( Result , S ) ; ContF ( error , E , _ S ) -&gt; E ( ) . Note that this solution does not throw away the call stack, as e.g. a call to hibernate/3 does. Thus, it is basically only tail-recursive as regards the calling function, placing plain_fsm:tail_apply/5 on the call stack rather than a function in the user module.","ref":"plain_fsm.html#tail_apply/5","title":"plain_fsm.tail_apply/5","type":"function"},{"doc":null,"ref":"plain_fsm.html#wake_up/5","title":"plain_fsm.wake_up/5","type":"function"},{"doc":null,"ref":"plain_fsm.html#t:cont/0","title":"plain_fsm.cont/0","type":"type"},{"doc":null,"ref":"plain_fsm.html#t:from/0","title":"plain_fsm.from/0","type":"type"},{"doc":null,"ref":"plain_fsm.html#t:misc/0","title":"plain_fsm.misc/0","type":"type"},{"doc":null,"ref":"plain_fsm.html#t:pdict/0","title":"plain_fsm.pdict/0","type":"type"},{"doc":null,"ref":"plain_fsm.html#t:sys_options/0","title":"plain_fsm.sys_options/0","type":"type"},{"doc":"Parse transform utility for plain_fsm","ref":"plain_fsm_xform.html","title":"plain_fsm_xform","type":"module"},{"doc":null,"ref":"plain_fsm_xform.html#format_error/1","title":"plain_fsm_xform.format_error/1","type":"function"},{"doc":null,"ref":"plain_fsm_xform.html#parse_transform/2","title":"plain_fsm_xform.parse_transform/2","type":"function"},{"doc":"# Plain FSM - Support library for wrigin textbook Erlang FSMs #\n\nCopyright (c) 2014-23 Ulf Wiger\n\n__Authors:__ [`ulf@wiger.net`](mailto:ulf@wiger.net).\n\nA behaviour/support library for writing plain Erlang FSMs.\n\nThis library implements an OTP behaviour for writing plain Erlang FSMs,\nalleviating a long-standing gripe of mine that the OTP behaviours, for all\ntheir power, force programmers into a coding style that is very much\ndifferent from that taught in the Basic Erlang Course (or the book, or\nonline tutorials, ...) -- the type of programming that made us want to\nuse Erlang in the first place.\n\nOnly in my old age have I begun to understand fully what a sacrifice\nthis is. See e.g. my presentation [Death by Accidental Complexity (QCon SF 2010)](http://www.infoq.com/presentations/Death-by-Accidental-Complexity)\nfor a more detailed discussion of the issues involved.\n(Slides also available in the `doc/` directory of this repos)\n\nThe requirements that drove us away from plain Erlang programming\nin the first place were:\n* __The need to support _system messages___ to control upgrade,\n   state inspection, shutdown, etc. The plain_fsm library solves this in a\n   reasonable way, I think.\n* __The need for debugging support__. The debugging support in\n   e.g. gen_server is, I believe, rendered obsolete by the new powerful\n   trace support (and dbg) in later versions of OTP.\n* In the case of gen_server, __reducing the need to reinvent the   wheel__, a valid point, but more so for e.g. the client side of\n   gen_server:call(). In a protocol state machine, the only thing that\n   really needs reusing is the handling of system messages.\n\n\n\nHowever, the behaviours provided by OTP for FSM programming,\n`gen_server` and `gen_fsm` (`gen_server`\nis perhaps a more common choice than `gen_fsm`), both have the\ndistinct drawback that you cannot normally start with a classic\nErlang design and then migrate to a behaviour without significant\ncode rewrite. In addition, the two behaviours are semantically different\nfrom the classic Erlang design","ref":"readme.html","title":"Plain FSM - Support library for wrigin textbook Erlang FSMs","type":"extras"},{"doc":"First, write your state machine without worrying about OTP system\nmessages. Once you're happy with it, figure out where you really want\nto handle system messages. Normally, it will suffice to do it in a fairly\nstable state. A good rule of thumb is that the top-level state machine\nshould handle system messages, while the transient (sub-) states\nshouldn't\n\nIn the states where you want to handle system messages, you have\nthree choices:\n\n\n### (A) Insert the system messages in the receive clause: ###\n\n```\n\nidle(S) ->\n   Parent = plain_fsm:info(parent),\n   receive\n      {system, From, Req} ->\n         plain_fsm:handle_system_msg(\n             Req, From, S, fun(S1) -> idle(S1) end);\n      {'EXIT', Parent, Reason} ->\n         plain_fsm:parent_EXIT(Reason, S);\n      ... %% your original code here\n   end.\n\n```\n\nThis has the advantage that everyone can understand what's going on.\nThe part that plain_fsm.erl helps you with is the set of functions\n`system_code_change()`, `system_continue()`,\n`system_shutdown()`, `format_status()`, which\nare required callbacks when you handle system messages directly.\n\n\n### (B) Handle system messages and unknown messages together: ###\n\n```\n\nidle(S) ->\n   Parent = plain_fsm:info(parent),\n   receive\n      ... %% your original code here\n      Msg ->\n         plain_fsm:handle_msg(Msg, State, fun(S1) -> idle(S1) end)\n   end.\n\n```\n\nThis is quite convenient if the receive statement already has a\n'catch-all' clause, discarding unknown messages.\n`plain_fsm:handle_msg/3` will handle system messages properly\nand ignore any other message.\n\n\n### (C) Write a pseudo wrapper function around your receive clause: ###\n\n```\n\nidle(S) ->\n   plain_fsm:extended_receive(\n      receive\n         ... %% your original code\n      end).\n\n```\n\nThe function `plain_fsm:extended_receive/1` is replaced\nin a _parse_transform_ into something that looks very much like\nthe previous program (A). The code, as it reads, requires the reader to\nknow that the transformation takes place, otherwise the semantics\nwould be confusing (you cannot solve the problem using a real function\nthat way.) On the plus side, this is a fairly small violation of both\nthe original code and Erlang's semantics.\n\n_Note that for this to work, you must include \"plain_fsm.hrl\"in your module._\n\n\n##","ref":"readme.html#using-plain_fsm","title":"Using plain_fsm ## - Plain FSM - Support library for wrigin textbook Erlang FSMs","type":"extras"},{"doc":"In the module [fsm_example.erl](https://github.com/uwiger/plain_fsm/blob/master/../src/fsm_example.erl)\n(included in the plain_fsm package), we choose to handle system\nmessages in the idle state. The example code is runnable, and supports\nsuspend, resume, status inspection, and code change.\n\nImagine that the code initially looked like this:\n\n\n```\n\nidle(S) ->\n    receive\n     a ->\n         io:format(\"going to state a~n\", []),\n         a(S);\n     b ->\n         io:format(\"going to state b~n\", []),\n         b(S)\n    after 10000 ->\n         io:format(\"timeout in idle~n\", []),\n         idle(S)\n    end).\n\n```\n\nThe change required to handle system messages is as follows:\n\n\n```\n\nidle(S) -> plain_fsm:extended_receive (\n      receive\n          a ->\n              io:format(\"going to state a~n\", []),\n              a(S);\n          b ->\n              io:format(\"going to state b~n\", []),\n              b(S)\n      after 10000 ->\n              io:format(\"timeout in idle~n\", []),\n              idle(S)\n      end).\n\n```\n\nIn addition, we change the start function from, in this case:\n\n\n```\n\nspawn_link() ->\n    spawn_link(fun() ->\n                       process_flag(trap_exit, true),\n                       idle(mystate)\n               end).\n\n```\n\n\nIs changed into:\n\n\n```\n\nspawn_link() -> plain_fsm:spawn_link (?MODULE, fun() ->\n                                          process_flag(trap_exit,true),\n                                          idle(mystate)\n                                  end).\n\n```\n\n\nSee also [spawn/2](https://github.com/uwiger/plain_fsm/blob/master/doc/README.md#spawn-2) and [spawn_opt/3](https://github.com/uwiger/plain_fsm/blob/master/doc/README.md#spawn_opt-3)\nfor information on other possible start functions.\n\nTo be fully compliant, you also need to supply a code_change/3 function.\nSee [behaviour_info/1](https://github.com/uwiger/plain_fsm/blob/master/doc/README.md#behaviour_info-1) for details.","ref":"readme.html#example","title":"Example #### - Plain FSM - Support library for wrigin textbook Erlang FSMs","type":"extras"},{"doc":"fsm_example   \n   plain_fsm   \n   plain_fsm_xform","ref":"readme.html#modules","title":"Modules ## - Plain FSM - Support library for wrigin textbook Erlang FSMs","type":"extras"},{"doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"Plain_fsm, copyright 2014-16 Ulf Wiger\n\nThis product contains code developed at\n- Ericsson AB (http://www.ericsson.com)\n- Erlang Solutions (http://www.erlang-solutions.com/)\n\nThe original was implemented by Ulf Wiger while at Ericsson AB,\nand released under the EPL license as part of http://jungerl.sf.net/\n\nCopyright transfered to Ulf Wiger 2014.","ref":"notice.html","title":"NOTICE","type":"extras"}]}